#define INCL_WINWINDOWMGR
#include <os2.h>
#include <gl.h>
#include <glu.h>
#include <aux.h>
#include <stdlib.h>

#include "glwindow.h"

#define FOVY            40.0
#define MINVIEWDISTANCE 100.0
#define MAXVIEWDISTANCE 1000.0

HGC hgc;

void cleanupOpenGL( HWND hWindow )
{
   HAB hab = WinQueryAnchorBlock( hWindow );

   /* unbind the current context */
   pglMakeCurrent( hab, NULL, NULL );

   /* destroy the context we created */
   pglDestroyContext( hab, hgc );
}

void initOpenGL( HWND hWindow )
{
   int attribList[] = { PGL_RGBA, PGL_RED_SIZE, 2, PGL_GREEN_SIZE, 2,
      PGL_BLUE_SIZE, 2, PGL_ALPHA_SIZE, 1, PGL_DOUBLEBUFFER, 0 };
   int val;
   /* get a handle to anchor block */
   HAB hab =  WinQueryAnchorBlock( hWindow );

   /* create a visual config */
   PVISUALCONFIG pVisualConfig = pglChooseConfig( hab, attribList );

   /* create a direct OpenGL context */
   hgc = pglCreateContext( hab, pVisualConfig, NULL, TRUE );

   /* and make the context we got the current one */
   val = pglMakeCurrent( hab, hgc, hWindow );

   /* enable depth buffer */
   glEnable( GL_DEPTH_TEST );

   glClearColor( 0.0, 0.0, 0.0, 1.0 );
   glDrawBuffer( GL_FRONT_AND_BACK );
   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT  );
   glDrawBuffer( GL_BACK );

   glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
   glShadeModel( GL_FLAT );

   /* free the visual config created above */
   free( pVisualConfig );
}

/* openGL settings */
void setWindowSize( int width, int height )
{
   /* setup the 3D perspective window */
   glMatrixMode( GL_PROJECTION );
   glLoadIdentity();
   gluPerspective( FOVY, (float)width / (float)height, MINVIEWDISTANCE, MAXVIEWDISTANCE );
   glMatrixMode( GL_MODELVIEW );
   /* make a viewport the acual size of the window */
   glViewport( 0, 0, width, height );
}

void paintWindow( HWND hWindow )
{
   /* clear the buffer */
   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

   /* no rotations */
   glLoadIdentity();

   /* move the image back from the camera */
   glTranslatef( 0.0, 0.0, -250.0 );
   auxWireSphere( 50.0 );

   /* swap the back draw buffer to the front */
   pglSwapBuffers( WinQueryAnchorBlock( hWindow ), hWindow );
}

/* message handling function */
MRESULT EXPENTRY ClientWindowProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
   HPS   hps = NULLHANDLE;

   switch( msg )
   {
      case WM_PAINT:
         /* paint routine */
         hps = WinBeginPaint( hwnd, NULLHANDLE, NULL );

         /* call our opengl paint function */
         paintWindow( hwnd );

         WinEndPaint( hps );
         break;

      case WM_SIZE:
         /* tell new window resize to opengl */
         setWindowSize(  SHORT1FROMMP( mp2 ),  SHORT2FROMMP( mp2 ) );
         break;

      default:
         return( WinDefWindowProc( hwnd, msg, mp1, mp2 ));
    }
    return( NULL );
}
